from abstract_security import *
from abstract_database import *
from .responseContentParser import *
import psycopg2
# Predefined table configurations
def connect_db():
    """Establish a connection to the database."""
    try:
        return psycopg2.connect(
            dbname=get_env_value(key="abstract_ai_dbname"),
            user=get_env_value(key="abstract_ai_user"),
            password=get_env_value(key="abstract_ai_password"),
            host=get_env_value(key="abstract_ai_host"),
            port=int(get_env_value(key="abstract_ai_port"))
        )
    except psycopg2.OperationalError as e:
        print("Unable to connect to the database:", e)
        sys.exit(1)
def get_table_configuration(file_path=None):
    table_configuration = []    
    table_configuration_file_path = file_path or get_env_value(key="abstract_ai_table_configuration_file_path")
    try:
        table_configuration = safe_read_from_json(table_configuration_file_path)
    except:
        print('no table config file path')
    return table_configuration
def get_dict_from_config(tableName,file_path=None):
    """Retrieve value from table configuration."""
    for config in get_table_configuration(file_path=file_path):
        if config.get('tableName').lower() == tableName.lower():
            return config
def get_table_names(file_path=None):
    return [config.get('tableName') for config in get_table_configuration(file_path=file_path)]
def get_first_row_as_dict(tableName=None):
    """Fetch the first row of data from the specified table and return as a dictionary."""
    tableName = tableName or get_env_value(key="abstract_ai_table_name")
    query = f"SELECT * FROM {tableName} ORDER BY id ASC LIMIT 1;"
    
    conn = connect_db()  # Assume connect_db() is your DB connection function
    cur = conn.cursor()
    try:
        cur.execute(query)
        first_row = cur.fetchone()  # Fetch the first row
        
        # Get column names from the cursor description
        col_names = [desc[0] for desc in cur.description]
        
        # Combine column names and row data into a dictionary
        if first_row:
            row_as_dict = dict(zip(col_names, first_row))
            return row_as_dict
        else:
            return None  # Return None if no rows are found

    except psycopg2.Error as e:
        print(f"Error fetching the first row: {e}")
        return None
    finally:
        cur.close()
        conn.close()

def get_instruction_from_tableName(tableName=None):
    tableName = tableName or get_env_value(key="abstract_ai_table_name")
    table_samples = []
    table_configuration_file_path = get_env_value(key="abstract_ai_table_configuration_file_path")
    try:
        table_configuration = safe_read_from_json(table_configuration_file_path)
        table_samples.append({"DATABASE_CONFIG":get_dict_from_config(tableName),"explination":"Database Table Configuration."})
    except:
        pass
    data = get_first_row_as_dict(tableName)
    table_samples.append({"ACTUAL_DATABASE_ROW":data,"explination":f"first row of data from table {tableName} returned as a dictionary."})
    value_keys = {}
    for column,values in data.items():
        value_keys[column]= get_value_keys(values)
    table_samples.append({"VALUE_KEYS":value_keys,"explination":"Type Values for the Values in the Database SCHEMA."})
    table_samples.append({"AVAILABLE_FUNCTION_FOR_FILTERING":f"""def search_multiple_fields(query):
    conn = connect_db()
    cur = conn.cursor()
    try:
        cur.execute(query)
        results = cur.fetchall()
        return results
    except psycopg2.Error as e:
        print(f"Error querying JSONB data: e")
    finally:
        cur.close()
        conn.close()
""","explination":"this is the available function for filtering the database"})
    return table_samples
def search_multiple_fields(query):
    conn = connect_db()
    cur = conn.cursor()
    try:
        cur.execute(query)
        results = cur.fetchall()
        return results
    except psycopg2.Error as e:
        print(f"Error querying JSONB data: {e}")
    finally:
        cur.close()
        conn.close()

def save_to_excel(rows, file_path="output.xlsx"):
    excel = []
    if rows:
        for i,row in enumerate(rows):
            if not isinstance(row,list):
                row = list(row)
            if isinstance(row,tuple):
                row = list(row)
    
            excel.append(flatten_json(row, parent_key='', sep='_'))
        safe_excel_save(pd.DataFrame(excel),file_path)
def get_query_save_to_excel(database_query,file_path ="output.xlsx"):
    result = search_multiple_fields(**database_query)
    save_to_excel(rows=result,file_path=file_path)
    return file_path
def generate_query_from_recent_response(file_path):
    response_content = get_response_content(file_path)
    database_query = response_content.get('database_query')
    if database_query:
        title = response_content.get('generate_title')
        dirName = os.path.dirname(file_path)
        parent_dir = os.path.dirname(dirName)
        new_directory = os.path.join(parent_dir,'queries')
        os.makedirs(new_directory,exist_ok=True)
        new_file_name = f"{title}.xlsx"
        new_file_path = os.path.join(new_directory,new_file_name)
        get_query_save_to_excel(database_query,file_path=new_file_path)
        return file_path



from .gpt_classes.api_selection.ApiBuilder import ApiManager
from .gpt_classes.model_selection.ModelBuilder import ModelManager
from .gpt_classes.prompt_selection.PromptBuilder import PromptManager
from .gpt_classes.response_selection.ResponseBuilder import ResponseManager
from .gpt_classes.instruction_selection.InstructionBuilder import InstructionManager
from .specializations import *
from .abstract_ai_gui import abstract_ai_gui



import os
import PySimpleGUIWeb as sg
from abstract_utilities import get_closest_match_from_list
from abstract_utilities.list_utils import ensure_nested_list,find_original_case
from abstract_utilities.class_utils import get_all_functions_for_instance,get_all_params,get_fun
class SimpleGuiFunctionsManager:
    def __init__(self):
        self.instance = sg
        self.function_names = get_all_functions_for_instance(instance=self.instance)
        self.function_names_dict = {}

    def _resolve_function_name(self, function_name):
        original_input = function_name
        # Check if the function name is already known
        if function_name in self.function_names_dict:
            return function_name
        for key,values in self.function_names_dict.items():
            if original_input in values["names"]:
                return key
        # Try to find the function name in the list of all functions
        if function_name not in self.function_names:
            function_name = find_original_case(self.function_names, function_name)
            if function_name == None:
                function_name = get_closest_match_from_list(comp_str=original_input, total_list=self.function_names)
                print(f"Function name: {original_input} resolved to closest match: {function_name}")
            else:
                print(f"Function name: {original_input} resolved to case match: {function_name}")
        # Add the resolved function name to the dictionary for future reference
        if function_name not in self.function_names_dict:
            all_params = self.get_all_parameters(function_name)
            self.function_names_dict[function_name] = {
                "names": [],
                "all_params": all_params,
                "required_params": all_params["names"]
            }
        if original_input not in self.function_names_dict[function_name]["names"]:
            self.function_names_dict[function_name]["names"].append(original_input)

        return function_name

    def get_all_parameters(self, function_name):
        return get_all_params(instance=self.instance, function_name=function_name)

    def get_gui_function(self, function_name: str = '', *args_2, args: dict = {}, **kwargs):
        function_name = self._resolve_function_name(function_name)
        fn_dict = self.function_names_dict[function_name]
        required_params = fn_dict["required_params"]

        # Map positional arguments to their respective required_params
        for i, param in enumerate(required_params):
            if i < len(args_2) and param not in args:
                args[param] = args_2[i]

        # Ensure default values for certain parameters
        for param in required_params:
            if param not in args:
                default_values = {"title": "", "layout": [[]], "values": [], "text": ""}
                if param in default_values:
                    args[param] = default_values[param]
                    if function_name != 'Window':
                        print(f"{param} for {function_name} not in arguments, using default {param} = {args[param]}")

        # Nested list for layout parameter
        if "layout" in fn_dict["all_params"]["names"]:
            args["layout"] = ensure_nested_list(args.get("layout", []))

        return get_fun({"instance": self.instance, "name": function_name, "args": args})

    def make_component(self, function_name, *args, **kwargs):
        if 'args' in kwargs:
            arges = kwargs['args']
            del kwargs['args']
            kwargs.update(arges)
        return self.get_gui_function(function_name, *args, args=kwargs)

class SimpleGuiFunctionsManagerSingleton:
    _instance = None
    @staticmethod
    def get_instance():
        if SimpleGuiFunctionsManagerSingleton._instance is None:
            SimpleGuiFunctionsManagerSingleton._instance = SimpleGuiFunctionsManager()
        return SimpleGuiFunctionsManagerSingleton._instance
def get_gui_fun(function_name: str = '', args: dict = {}):
    """
    Returns a callable object for a specific PySimpleGUI function with the provided arguments.

    Args:
        function_name (str): The name of the PySimpleGUI function.
        *args_2: Variable-length positional arguments.
        args (dict): The arguments to pass to the PySimpleGUI function.
        **kwargs: Variable-length keyword arguments.

    Returns:
        callable: A callable object that invokes the PySimpleGUI function with the specified arguments when called.
    """
    simple_gui_fun_mgr = SimpleGuiFunctionsManagerSingleton.get_instance()
    return simple_gui_fun_mgr.get_gui_function(function_name=function_name,args=args)

def make_component(function_name,*args,**kwargs):
    simple_gui_fun_mgr = SimpleGuiFunctionsManagerSingleton.get_instance()
    return simple_gui_fun_mgr.make_component(function_name,*args,**kwargs)


from abstract_gui import AbstractWindowManager,AbstractBrowser,text_to_key,ensure_nested_list,expandable,RightClickManager,get_screen_dimensions
right_click_mgr=RightClickManager()
def get_standard_screen_dimensions(width=0.70,height=0.80):
    return get_screen_dimensions(width=width,height=height)
window_width,window_height=get_standard_screen_dimensions()
def get_left_right_nav(name,section=True,push=True):
    insert = f"{name} {'section ' if section else ''}"
    nav_bar = [make_component("Button",button_text="<-",key=text_to_key(f"{insert}back"),enable_events=True),
         make_component("input",default_text='0',key=text_to_key(f"{insert}number"),size=(4,1)),
         make_component("Button",button_text="->",key=text_to_key(f"{insert}forward"),enable_events=True)]
    if push:
        nav_bar=[make_component("Push")]+nav_bar+[make_component("Push")]
    return nav_bar
def generate_bool_text(title:str,args:dict={})->object:
    return make_component("Frame",title, layout=[[get_right_click_multi(key=text_to_key(text=title,section='text'),args=args)]],**expandable())
def get_tab_layout(title:str,layout:list=None)->object:
    if not layout:
        layout = get_right_click_multi(key=text_to_key(title),args={**expandable(size=(None, 5))})
    return make_component("Tab",title.upper(),ensure_nested_list(layout))
def get_column(layout:list,args:dict={})->object:
    return make_component("Column",ensure_nested_list(layout),**args)
def get_tab_group(grouped_tabs:list,args:dict={})->object:
    return make_component("TabGroup",ensure_nested_list(grouped_tabs),**args)
def get_right_click_multi(key:str,args:dict={})->object:
    return make_component("Multiline",**args,right_click_menu=right_click_mgr.get_right_click(key=key),key=key)
completion_token_keys = ['completion tokens available','completion tokens desired','completion tokens used']
prompt_token_keys = ['prompt tokens available','prompt tokens desired','prompt tokens used']
chunk_data_keys=['max chunk size','chunk length','chunk total']
all_token_keys = completion_token_keys+prompt_token_keys+chunk_data_keys+['chunk sectioned data']
response_types = ['instruction', 'json', 'bash', 'text']
model_type_keys=['role','response type']
percentage_keys = ['prompt percentage','completion percentage']
file_options_keys = ['auto chunk title','reuse chunk data','append chunks','scan mode all']
test_options_keys = ['test run','test files','test file input','test browse']
instructions_keys = ["instructions","additional_responses","suggestions","abort","database_query","notation","generate_title","additional_instruction","request_chunks","prompt_as_previous","token_adjustment"]
api_keys = ["header",'api key','api env']
